# -*- coding: utf-8 -*-
"""
This package allows to access data from the Open Access Monitor (OAM),
which is run by Forschungszentrum Jülich (c) 2025.

For the OAM dashboard, see
https://open-access-monitor.de

For the OAM documentation, see
https://jugit.fz-juelich.de/synoa/oam-dokumentation/-/wikis/home
"""

__author__ = "Donatus Herre <donatus.herre@slub-dresden.de>"
__version__ = "0.3.0"

import os

try:
    EMAIL = os.environ["OAM_EMAIL"]
except KeyError:
    import socket
    import getpass
    host = socket.gethostname()
    user = getpass.getuser()
    EMAIL = "{0}@{1}".format(user, host)

try:
    TOKEN = os.environ["OAM_TOKEN"]
except KeyError:
    TOKEN = None

from . import client, query


def get_client(headers={}, token=""):
    if not token and TOKEN:
        token = TOKEN
    return client.OpenAccessMonitorAPI(headers=headers, token=token)


def get_journal(issn, headers={}):
    oamapi = get_client(headers=headers)
    return oamapi.journal(issn)


def get_publication(doi, headers={}):
    oamapi = get_client(headers=headers)
    return oamapi.publication(doi)


def get_publication_costs(doi, headers={}):
    oamapi = get_client(headers=headers)
    return oamapi.publication_costs(doi)


def run_search(find, limit=10, scroll=False, headers={}, **kwargs):
    oamapi = get_client(headers=headers)
    if scroll:
        return oamapi.scroll(find, limit=limit, **kwargs)
    return oamapi.search(find, limit=limit, **kwargs)


def run_journal_search(limit=10, scroll=True, headers={}, **kwargs):
    result = run_search("Journals", limit=limit, scroll=scroll, headers=headers, **kwargs)
    if result and len(result) > 0:
        return [client.docs.JournalParser(j) for j in result]


def get_agreement_journals(agreement, limit=10, scroll=True, filter={}, headers={}, **kwargs):
    agreement_filter = query.filter_agreements(agreement, filter=filter)
    return run_journal_search(limit=limit, scroll=scroll, headers=headers, filter=agreement_filter, **kwargs)


def run_publication_search(limit=10, scroll=False, headers={}, **kwargs):
    result = run_search("Publications", limit=limit, scroll=scroll, headers=headers, **kwargs)
    if result and len(result) > 0:
        return [client.docs.PublicationParser(p) for p in result]


def get_wos_grid_publications(grid_id, limit=10, scroll=False, filter={}, headers={}, **kwargs):
    grid_filter = query.filter_wos_organisation_grid(grid_id, filter=filter)
    return run_publication_search(limit=limit, scroll=scroll, headers=headers, filter=grid_filter, **kwargs)


def get_wos_grid_latest_publications(grid_id, limit=10, scroll=False, filter={}, headers={}, **kwargs):
    latest_sort = query.sort_desc("published_date")
    return get_wos_grid_publications(grid_id, limit=limit, scroll=scroll, filter=filter, headers=headers,
                                     sort=latest_sort, **kwargs)


def get_dim_ror_publications(ror_id, limit=10, scroll=False, filter={}, headers={}, **kwargs):
    ror_filter = query.filter_dim_organisation_ror(ror_id, filter=filter)
    return run_publication_search(limit=limit, scroll=scroll, headers=headers, filter=ror_filter, **kwargs)


def get_dim_ror_latest_publications(ror_id, limit=10, scroll=False, filter={}, headers={}, **kwargs):
    latest_sort = query.sort_desc("published_date")
    return get_dim_ror_publications(ror_id, limit=limit, scroll=scroll, filter=filter, headers=headers,
                                    sort=latest_sort, **kwargs)


def get_wos_ror_publications(ror_id, limit=10, scroll=False, filter={}, headers={}, **kwargs):
    ror_filter = query.filter_wos_organisation_ror(ror_id, filter=filter)
    return run_publication_search(limit=limit, scroll=scroll, headers=headers, filter=ror_filter, **kwargs)


def get_wos_ror_latest_publications(ror_id, limit=10, scroll=False, filter={}, headers={}, **kwargs):
    latest_sort = query.sort_desc("published_date")
    return get_wos_ror_publications(ror_id, limit=limit, scroll=scroll, filter=filter, headers=headers,
                                    sort=latest_sort, **kwargs)


def get_dim_grid_publications(grid_id, limit=10, scroll=False, filter={}, headers={}, **kwargs):
    grid_filter = query.filter_dim_organisation_grid(grid_id, filter=filter)
    return run_publication_search(limit=limit, scroll=scroll, headers=headers, filter=grid_filter, **kwargs)


def get_dim_grid_latest_publications(grid_id, limit=10, scroll=False, filter={}, headers={}, **kwargs):
    latest_sort = query.sort_desc("published_date")
    return get_dim_grid_publications(grid_id, limit=limit, scroll=scroll, filter=filter, headers=headers,
                                     sort=latest_sort, **kwargs)


def run_publication_cost_search(limit=10, scroll=False, headers={}, **kwargs):
    result = run_search("PublicationCosts", limit=limit, scroll=scroll, headers=headers, **kwargs)
    if result and len(result) > 0:
        return [client.docs.PublicationCostsParser(pc) for pc in result]


def get_openapc_grid_publication_cost(grid_id, limit=10, scroll=False, filter={}, headers={}, **kwargs):
    grid_filter = query.filter_openapc_organisation_grid(grid_id, filter=filter)
    return run_publication_cost_search(limit=limit, scroll=scroll, headers=headers, filter=grid_filter, **kwargs)


def get_openapc_ror_publication_cost(ror_id, limit=10, scroll=False, filter={}, headers={}, **kwargs):
    ror_filter = query.filter_openapc_organisation_ror(ror_id, filter=filter)
    return run_publication_cost_search(limit=limit, scroll=scroll, headers=headers, filter=ror_filter, **kwargs)



# -*- coding: utf-8 -*-
"""
Parser classes for data retrieved from the Open Access Monitor (OAM)

For more information on the database (i.e. MongoDB) schema of OAM, see
https://jugit.fz-juelich.de/synoa/oam-dokumentation/-/wikis/English-Version/Open-Access-Monitor/Database-Schema
"""


class BaseParser:

    def __init__(self, data):
        self.raw = data

    def _names(self):
        if self.raw:
            names = list(self.raw.keys())
            names.sort()
            return names
        return []

    def _field(self, name):
        if self.raw and name in self.raw:
            return self.raw[name]

    def get(self, name):
        return self._field(name)


class RorAddressParser(BaseParser):

    def __init__(self, data):
        super().__init__(data)

    @property
    def city(self):
        return self._field("city")

    @property
    def state(self):
        return self._field("state")

    @property
    def state_code(self):
        return self._field("state_code")

    @property
    def country(self):
        return self._field("country")

    @property
    def country_code(self):
        return self._field("country_code")

    @property
    def lat(self):
        return self._field("lat")

    @property
    def lng(self):
        return self._field("lng")

    @property
    def primary(self):
        return self._field("primary")

    @property
    def postcode(self):
        return self._field("postcode")


class RorLabelParser(BaseParser):

    def __init__(self, data):
        super().__init__(data)

    @property
    def label(self):
        return self._field("label")

    @property
    def iso639(self):
        return self._field("iso639")


class RorRelationshipParser(BaseParser):

    def __init__(self, data):
        super().__init__(data)

    @property
    def type(self):
        return self._field("type")

    @property
    def id(self):
        return self._field("id")

    @property
    def included(self):
        return self._field("included")


class ObjectParser(BaseParser):

    def __init__(self, data):
        super().__init__(data)

    @property
    def id(self):
        return self._field("_id")


class OrganisationParser(ObjectParser):

    def __init__(self, data):
        super().__init__(data)

    @property
    def grid_id(self):
        return self._field("grid_id")

    @property
    def name(self):
        return self._field("name")

    @property
    def aliases(self):
        return self._field("aliases")

    @property
    def acronyms(self):
        return self._field("acronyms")

    @property
    def type(self):
        return self._field("type")

    @property
    def _address(self):
        return self._field("address")

    @property
    def address(self):
        field = self._address
        if field:
            return RorAddressParser(field)

    @property
    def _labels(self):
        return self._field("labels")

    @property
    def labels(self):
        fields = self._labels
        if isinstance(fields, list) and len(fields) > 0:
            return [RorLabelParser(lab) for lab in fields]

    @property
    def _relationships(self):
        return self._field("relationships")

    @property
    def relationships(self):
        fields = self._relationships
        if isinstance(fields, list) and len(fields) > 0:
            return [RorRelationshipParser(r) for r in fields]

    @property
    def corresponding(self):
        return self._field("corresponding")


class PublicationSourceDataParser(ObjectParser):

    def __init__(self, data):
        super().__init__(data)

    @property
    def _organisations(self):
        return self._field("organisations")

    @property
    def organisations(self):
        fields = self._organisations
        if isinstance(fields, list) and len(fields) > 0:
            return [OrganisationParser(o) for o in fields]

    @property
    def source_id(self):
        return self._field("source_id")

    @property
    def citation_count(self):
        return self._field("citation_count")

    @property
    def url(self):
        return self._field("url")


class OaObjectParser(ObjectParser):

    def __init__(self, data):
        super().__init__(data)

    @property
    def oa_color(self):
        return self._field("oa_color")


class PublisherParser(OaObjectParser):

    def __init__(self, data):
        super().__init__(data)

    @property
    def name(self):
        return self._field("name")


class JournalParser(OaObjectParser):

    def __init__(self, data):
        super().__init__(data)
        self._delim = "|"

    @property
    def title(self):
        return self._field("title")

    @property
    def issns(self):
        return self._field("issns")

    def get_issns(self, joined=True):
        issns = self.issns
        if isinstance(issns, list) and len(issns) > 0:
            if joined:
                return self._delim.join(issns)
            return issns

    @property
    def flags(self):
        return self._field("flags")

    def get_flags(self, joined=True):
        flags = self.flags
        if isinstance(flags, list) and len(flags) > 0:
            if joined:
                return self._delim.join(flags)
            return flags

    @property
    def agreements(self):
        return self._field("agreements")

    def get_agreements(self, joined=True):
        agreements = self.agreements
        if isinstance(agreements, list) and len(agreements) > 0:
            if joined:
                return self._delim.join([a["_id"] for a in agreements])
            return agreements

    @property
    def csv_header(self):
        return ["id", "title", "oa_color", "issns", "agreements"]

    @property
    def csv_row(self):
        return [self.id, self.title,
                self.oa_color,
                self.get_issns() or "",
                self.get_agreements() or ""]


class PubObjectParser(OaObjectParser):

    def __init__(self, data):
        super().__init__(data)

    @property
    def _journal(self):
        return self._field("journal")

    @property
    def journal(self):
        field = self._journal
        if field:
            return JournalParser(field)

    @property
    def _publisher(self):
        return self._field("publisher")

    @property
    def publisher(self):
        field = self._publisher
        if field:
            return PublisherParser(field)

    @property
    def published_date(self):
        return self._field("published_date")


class PublicationParser(PubObjectParser):

    def __init__(self, data):
        super().__init__(data)

    @property
    def year(self):
        return self._field("year")

    @property
    def _dim(self):
        return self._field("dim")

    @property
    def dim(self):
        field = self._dim
        if field:
            return PublicationSourceDataParser(field)

    @property
    def _wos(self):
        return self._field("wos")

    @property
    def wos(self):
        field = self._wos
        if field:
            return PublicationSourceDataParser(field)


class PublicationCostsParser(PubObjectParser):

    def __init__(self, data):
        super().__init__(data)

    # @property
    # def _organisations(self):
    #     return self._field("organisations")

    # @property
    # def organisations(self):
    #     fields = self._organisations
    #     if isinstance(fields, list) and len(fields) > 0:
    #         return [OrganisationParser(o) for o in fields]

    @property
    def doi(self):
        return self._field("doi")

    # @property
    # def year(self):
    #     return self._field("year")

    # @property
    # def oa_charges(self):
    #     return self._field("oa_color")

    # @property
    # def apc(self):
    #     return self._field("apc")

    # @property
    # def color_charges(self):
    #     return self._field("color_charges")

    # @property
    # def cover(self):
    #     return self._field("cover")

    # @property
    # def hybrid_oa(self):
    #     return self._field("hybrid_oa")

    # @property
    # def other(self):
    #     return self._field("other")

    # @property
    # def page_charges(self):
    #     return self._field("page_charges")

    # @property
    # def publication_charges(self):
    #     return self._field("publication_charges")

    # @property
    # def reprint(self):
    #     return self._field("reprint")

    # @property
    # def submission_fee(self):
    #     return self._field("submission_fee")

    # @property
    # def total(self):
    #     return self._field("total")

    @property
    def _openapc(self):
        return self._field("openapc")

    @property
    def openapc(self):
        field = self._openapc
        if field:
            return OpenApcParser(field)


class OpenApcParser(BaseParser):

    def __init__(self, data):
        super().__init__(data)

    @property
    def _organisations(self):
        return self._field("organisations")

    @property
    def organisations(self):
        fields = self._organisations
        if isinstance(fields, list) and len(fields) > 0:
            return [OrganisationParser(o) for o in fields]

    @property
    def year(self):
        return self._field("year")

    @property
    def apc(self):
        return self._field("apc")

    @property
    def total(self):
        return self._field("total")



import click
from . import get_publication, get_journal
from .utils import json_str, json_str_pretty

from . import __version__, EMAIL

HEADERS = {
    "User-Agent": "oampy-cli {0} <{1}>".format(__version__, EMAIL)}


def json_output(raw, pretty):
    if pretty:
        return json_str_pretty(raw)
    else:
        return json_str(raw)


def print_json(raw, pretty):
    if raw is not None:
        print(json_output(raw, pretty))


def print_result(result, pretty):
    if result is not None:
        print_json(result.raw, pretty)


@click.group()
def main():
    """
    Fetch metadata from the Open Access Monitor (OAM)
    """
    pass


def main_options(function):
    """
    Reuse decorators for multiple commands
    https://stackoverflow.com/a/50061489
    """
    function = click.option("-p", "--pretty", is_flag=True,
                            help="Pretty print output")(function)
    return function


@main.command()
@main_options
@click.argument('doi')
def publication(doi, pretty):
    """Fetch metadata of publication given by DOI"""
    p = get_publication(doi, headers=HEADERS)
    print_result(p, pretty)


@main.command()
@main_options
@click.argument('issn')
def journal(issn, pretty):
    """Fetch metadata of journal given by ISSN"""
    j = get_journal(issn, headers=HEADERS)
    print_result(j, pretty)


if __name__ == '__main__':
    main()



"""
Build queries for MongoDB-based Open Access Monitor API

For more information on the database commands, see
https://docs.mongodb.com/manual/reference/command/
"""


def filter_year(value, filter={}):
    filter["year"] = value
    return filter


def filter_year_from(value, filter={}):
    filter["year"] = {
      "$gte": value
    }
    return filter


def filter_year_until(value, filter={}):
    filter["year"] = {
      "$lte": value
    }
    return filter


def filter_published_date(value, filter={}):
    filter["published_date"] = value
    return filter


def filter_published_date_from(value, filter={}):
    filter["published_date"] = {
      "$gte": value
    }
    return filter


def filter_published_date_until(value, filter={}):
    filter["published_date"] = {
      "$lte": value
    }
    return filter


def filter_dim_organisation_ror(ror_id, filter={}):
    filter["dim.organisations._id"] = "https://ror.org/{0}".format(ror_id)
    return filter


def filter_dim_organisation_grid(grid_id, filter={}):
    filter["dim.organisations.grid_id"] = grid_id
    return filter


def filter_wos_organisation_ror(ror_id, filter={}):
    filter["wos.organisations._id"] = "https://ror.org/{0}".format(ror_id)
    return filter


def filter_wos_organisation_grid(grid_id, filter={}):
    filter["wos.organisations.grid_id"] = grid_id
    return filter


def filter_openapc_organisation_ror(ror_id, filter={}):
    filter["openapc.organisations._id"] = "https://ror.org/{0}".format(ror_id)
    return filter


def filter_openapc_organisation_grid(grid_id, filter={}):
    filter["openapc.organisations.grid_id"] = grid_id
    return filter


def filter_agreements(agreement, filter={}):
    filter["agreements._id"] = agreement
    return filter


def sort_asc(value):
    return {value: 1}


def sort_desc(value):
    return {value: -1}



import json
import logging
import requests

from . import __version__, EMAIL


def get_logger(name="oampy", loglevel=None):
    logger = logging.getLogger(name)
    if not logger.handlers:
        stream = logging.StreamHandler()
        if loglevel is not None and loglevel != stream.level:
            stream.setLevel(loglevel)
        stream.setFormatter(logging.Formatter(
            "%(asctime)s [%(levelname)s] %(name)s: %(message)s", "%Y-%m-%d %H:%M:%S"))
        logger.addHandler(stream)
        if loglevel is not None and loglevel != logger.level:
            logger.setLevel(loglevel)
    else:
        if loglevel is not None and logger.level != loglevel:
            logger.setLevel(loglevel)
    return logger


def get_request(url, params={}, headers={}):
    if "User-Agent" not in headers:
        headers["User-Agent"] = "oampy {0} <{1}>".format(__version__, EMAIL)
    try:
        return requests.get(url, params=params, headers=headers)
    except requests.exceptions.RequestException as err:
        logger = get_logger()
        logger.error(err.__class__.__name__)
        return None


def response_ok(response, loglevel=None):
    if response is None:
        return False
    if response.status_code == 200:
        return True
    else:
        logger = get_logger(loglevel=loglevel)
        logger.error("HTTP request to {0} failed!".format(response.url))
        logger.error("HTTP response code is {0}.".format(response.status_code))
        return False


def response_json(response):
    if response_ok(response):
        try:
            return response.json()
        except requests.exceptions.JSONDecodeError:
            logger = get_logger()
            logger.error(
                "Failed to parse JSON data retrieved from URL {0}".format(response.url))
            if response.text:
                logger.error("Found payload: {0}".format(response.text))


def json_request(url, headers={}):
    response = get_request(url, headers=headers)
    return response_json(response)


def data_ok(response):
    if response and response["ok"] == 1:
        return True
    return False


def oam_request(url, headers={}):
    response = json_request(url, headers=headers)
    if data_ok(response):
        return response


def oam_batch(response):
    if "cursor" in response:
        if "batch" in response["cursor"]:
            return response["cursor"]["batch"]


def json_str(data):
    return json.dumps(data, ensure_ascii=False, separators=(',', ':'))


def json_str_pretty(data):
    return json.dumps(data, ensure_ascii=False, indent=2)
